/*
 * SingleUserKinect takes care of tracking 1 user
 * When a user is tracked SingleUserKinect will ignore all other users.
 * When the user is out of View it will pick another one.
 */

class SingleUserKinect {

  PApplet app; 

  SimpleOpenNI context;

  ArrayList<Integer> validUsers = new ArrayList<Integer>(); 
  ArrayList<Integer> invalidUsers = new ArrayList<Integer>();
  int trackedUserId = 0;
  PVector pos = new PVector();
  float minZ = 50; //ignore people with a z position smaller then this value
  float maxZ = 4000; //ignore people with a z-position beyond this value 


  // Instantiate the SingleUserKinect
  public SingleUserKinect(PApplet app) {
    this.app = app;
    setupKinect();
  }

  // Sets up the kinect
  void setupKinect() {
    context = new SimpleOpenNI(app);
    if (context.isInit() == false)
    {
      println("Can't init SimpleOpenNI, maybe the camera is not connected!"); 
      exit();
      return;
    }
    // enable mirror
    context.setMirror(true);
    // enable depthMap generation 
    context.enableDepth();
    // enable skeleton generation for all joints
    context.enableUser();
  }

  // update user position
  void update() {
    context.update();
    updateUser();
  }


  // get a users Center of Mass
  void getCoM(PVector userPosRealWorld) {
    context.getCoM(trackedUserId, userPosRealWorld);
  }

  // make sure we are tracking a valid user
  void updateUser() {

    // check if our valid users are still valid. If they are not, move them to the list of
    // invalid users, and stop tracking him
    Iterator<Integer> iterator = validUsers.iterator();
    while (iterator.hasNext ()) {
      int userId = iterator.next();   
      context.getCoM(userId, pos);
      if (!userStillValid(pos.z)) {
        iterator.remove();
        invalidUsers.add(userId);
        if (userId == trackedUserId) {
          trackedUserId = 0;
        }
      }
    }


    // check if any invalid users are valid now. If they are, move them to the list of
    // valid users.
    iterator = invalidUsers.iterator();
    while (iterator.hasNext ()) {
      int userId = iterator.next();   
      context.getCoM(userId, pos);
      if (userStillValid(pos.z)) {
        iterator.remove();
        validUsers.add(userId);
      }
    }

    // start tracking the first user in the list of valid users.
    for (int userId: validUsers) {
      if (userId != trackedUserId) {
        trackUser(context, userId);
      }
      break;
    }
  }

  // if the user is too close or too far he isn't valid anymore.
  // (the kinect gives a z value of 0 when someone is too close, or when they exited,
  // while still holding on to )
  boolean userStillValid (float posZ) {
    if (posZ > minZ && posZ < maxZ) {
      return true;
    }
    return false;
  }

  // start tracking this user
  void trackUser(SimpleOpenNI curContext, int userId) {
    println("startTracking - userId: " + userId);
    //curContext.startTrackingSkeleton(userId);
    kinect.trackedUserId = userId;
  }

  // add this user to our list, and start tracking this user if we weren't tracking someone already
  void registerNewUser(SimpleOpenNI curContext, int userId) {
    println("onNewUser - userId: " + userId);
    invalidUsers.add(userId);
    //if (trackedUserId == 0)
    //  trackUser(curContext, userId);
  }

  // remove this user from our lists, and stop tracking him 
  void deRegisterUser(SimpleOpenNI curContext, int userId) {
    println("onLostUser - userId: " + userId);
    while (kinect.validUsers.indexOf (userId) > -1) {
      validUsers.remove(validUsers.indexOf(userId));
    }
    while (kinect.invalidUsers.indexOf (userId) > -1) {
      invalidUsers.remove(invalidUsers.indexOf(userId));
    }
    if (userId == trackedUserId) {
      trackedUserId = 0;
    }
  }
}

